﻿using FunnyCommon.Shared;

namespace FunnyCommon.Local
{
    public class PiGpoutManager : IPiGpoutManager
    {
        private readonly PiFileGpout[] _arr;

        public PiGpoutManager(int maxNum)
        {
            _arr = new PiFileGpout[maxNum];
        }

        /// <summary>
        /// true: rob, false: norm, null: dy
        /// </summary>
        public bool? Allocate(byte pin)
        {
            if (_arr[pin] != null) return null; // Already allocated

            var rob = PiFileGpout.Rob(pin);

            _arr[pin] = new PiFileGpout(pin);

            return rob;
        }

        /// <summary>
        /// false: dy, true: norm
        /// </summary>
        public bool Free(byte pin)
        {
            if (_arr[pin] == null) return false; // Already free
            _arr[pin].Dispose();
            _arr[pin] = null;
            return true;
        }

        public void WritePin(byte pin, bool height)
        {
            _arr[pin].WritePin(height);
        }

        public void Pulse(byte pin, bool height, int? intervalMs = null)
        {
            _arr[pin].Pulse(height, intervalMs);
        }

        public void Dispose()
        {
            foreach (var piFileGpout in _arr)
            {
                piFileGpout?.Dispose();
            }
        }
    }
}