﻿using System;
using System.Collections.Generic;
using FunnyCommon.Remoting.Interfaces;
using FunnyCommon.Remoting.Protocol;
using FunnyCommon.Shared;

namespace FunnyCommon.Remoting.Local
{
    public class LocalGpoutManager : IRemoteGpoutManager
    {
        private readonly IPiGpoutManager _gpoutManager;

        private class DummyDisposable : IDisposable
        {
            public void Dispose()
            {
            }
        }

        public LocalGpoutManager(IPiGpoutManager gpoutManager)
        {
            _gpoutManager = gpoutManager;
        }

        public void Dispose()
        {
            _gpoutManager.Dispose();
        }

        public bool? Allocate(byte pin)
        {
            return _gpoutManager.Allocate(pin);
        }

        public bool Free(byte pin)
        {
            return _gpoutManager.Free(pin);
        }

        public void WritePin(byte pin, bool height)
        {
            _gpoutManager.WritePin(pin, height);
        }

        public Response SendCommand(Command command)
        {
            return command.Execute(_gpoutManager);
        }

        public Response[] SendCommandBulk(Command[] commands)
        {
            var lstResp = new List<Response>(commands.Length);
            foreach (var command in commands)
            {
                try
                {
                    lstResp.Add(command.Execute(_gpoutManager));
                }
                catch (Exception ex)
                {
                    lstResp.Add(new Response { Success = false, });// Message = ex.Message });
                    break;
                }
            }
            return lstResp.ToArray();
        }

        public IDisposable CreateBulk(int estimateMaxCount)
        {
            return new DummyDisposable();
        }
    }
}