﻿using System;
using System.Linq;
using FunnyCommon.Remoting.Interfaces;
using FunnyCommon.Remoting.Protocol;
using FunnyCommon.Shared;

namespace FunnyLcdJlx1770063
{
    public class Jlx1770063Driver
    {
        private readonly byte _pinRd;
        private readonly byte _pinWr;
        private readonly byte _pinRs;
        private readonly byte _pinCs;
        private readonly byte _pinLeda;
        private readonly byte _pinRst;
        private readonly byte[] _pinData;

        private readonly IRemoteGpoutManager _manager;
        private IDisposable _bulk;

        public Jlx1770063Driver(byte pinRd, byte pinWr, byte pinRs, byte pinCs,
            byte pinLeda, byte pinRst, byte[] pinData, IRemoteGpoutManager manager)
        {
            _manager = manager;
            if (pinData.Length != 8) throw new ArgumentException(nameof(pinData));

            _pinRd = pinRd;
            _pinWr = pinWr;
            _pinRs = pinRs;
            _pinCs = pinCs;
            _pinLeda = pinLeda;
            _pinRst = pinRst;
            _pinData = pinData.ToArray();

            Flush();

            _manager.Allocate(_pinRd);
            _manager.Allocate(_pinWr);
            _manager.Allocate(_pinRs);
            _manager.Allocate(_pinCs);
            _manager.Allocate(_pinLeda);
            _manager.Allocate(_pinRst);

            foreach (var b in _pinData)
            {
                _manager.Allocate(b);
            }

            Flush();

            Init();

            Flush();
        }

        public void Flush()
        {
            _bulk?.Dispose();
            _bulk = _manager.CreateBulk();
        }

        private void Init()
        {
            //RESET
            _manager.SendCommand(new Command { Type = CommandType.Sleep, Value = 50 });
            _manager.WritePin(_pinRst, 0);
            _manager.SendCommand(new Command { Type = CommandType.Sleep, Value = 50 });
            _manager.WritePin(_pinRst, 1);
            _manager.SendCommand(new Command { Type = CommandType.Sleep, Value = 10 });

            //开始初始化：
            WriteCom(0x11);
            WriteCom(0xb1);
            WriteData(0x01);
            WriteData(0x2c);
            WriteData(0x2d);
            WriteCom(0xb2);
            WriteData(0x01);
            WriteData(0x2c);
            WriteData(0x2d);
            WriteCom(0xb3);
            WriteData(0x01);
            WriteData(0x2c);
            WriteData(0x2d);
            WriteData(0x01);
            WriteData(0x2d);
            WriteData(0x2d);
            WriteCom(0xb4);
            WriteData(0x02);
            WriteCom(0xb6);
            WriteData(0xb4);
            WriteData(0xf0);
            WriteCom(0xc0);
            WriteData(0xa2);
            WriteData(0x02);
            WriteData(0x84);
            WriteCom(0xc1);
            WriteData(0xc5);
            WriteCom(0xc2);
            WriteData(0x0a);
            WriteData(0x00);
            WriteCom(0xc3);
            WriteData(0x8a);
            WriteData(0x2a);
            WriteCom(0xc4);
            WriteData(0x8a);
            WriteData(0xee);
            WriteCom(0xc5);
            WriteData(0x0e);
            WriteCom(0x36); //行扫描顺序，列扫描顺序，横放/竖放
            WriteData(0xC8); //MX=1（行地址顺序：从左到右），MY=1（列地址顺序：从上到下），MV=0（竖放），ML=0(纵向刷新：从上到下），RGB = 1（依次为RGB），MH = 0（横向刷新顺序：从左到右）
            WriteCom(0xff);
            WriteData(0x40);
            WriteData(0x03);
            WriteData(0x1a);
            WriteCom(0xfc);
            WriteData(0x11);
            WriteData(0x17);
            WriteCom(0xf0);
            WriteData(0x01);
            WriteCom(0x3a);
            WriteData(0x05);
            WriteCom(0xf6);
            WriteData(0x00);
            WriteCom(0xe0);
            WriteData(0x02);
            WriteData(0x1c);
            WriteData(0x07);
            WriteData(0x12);
            WriteData(0x37);
            WriteData(0x32);
            WriteData(0x29);
            WriteData(0x2d);
            WriteData(0x29);
            WriteData(0x25);
            WriteData(0x2b);
            WriteData(0x39);
            WriteData(0x00);
            WriteData(0x01);
            WriteData(0x03);
            WriteData(0x10);
            WriteCom(0xe1);
            WriteData(0x0b);
            WriteData(0x14);
            WriteData(0x09);
            WriteData(0x26);
            WriteData(0x27);
            WriteData(0x22);
            WriteData(0x1c);
            WriteData(0x20);
            WriteData(0x1d);
            WriteData(0x1a);
            WriteData(0x25);
            WriteData(0x2d);
            WriteData(0x06);
            WriteData(0x06);
            WriteData(0x02);
            WriteData(0x0f);
            //WriteCom(0x2a); //定义X 地址的开始及结束位置
            //WriteData(0x00);
            //WriteData(0x00);
            //WriteData(0x00);
            //WriteData(0x7F);
            //WriteCom(0x2b); //定义Y 地址的开始及结束位置
            //WriteData(0x00);
            //WriteData(0x00);
            //WriteData(0x00);
            //WriteData(0x9F);
            WriteCom(0x29); //开显示
        }

        public void FillColor(ushort xb, ushort yb, ushort xl, ushort yl, ushort color)
        {
            AddrSet(xb, yb, xl, yl);
            for (var x = 0; x < xl; x++)
            {
                for (var y = 0; y < yl; y++)
                {
                    WriteData16(color);
                }
            }
        }

        public void FillColor(ushort xb, ushort yb, ushort xl, ushort yl, ushort[] data)
        {
            AddrSet(xb, yb, xl, yl);
            for (var x = 0; x < xl * yl; x++)
            {
                WriteData16(data[x]);
            }
            Flush();
        }

        public void SetLed(bool on)
        {
            _manager.WritePin(_pinLeda, on);
        }

        private void AddrSet(ushort x, ushort y, ushort w, ushort h)
        {
            //定义窗口坐标：开始坐标(x,y)以及窗口大小(w,h)

            var xe = (ushort)(x + w - 1);
            var ye = (ushort)(y + h - 1);
            WriteCom(0x2a); // 设置X 开始及结束的地址
            WriteData16(x); // X 开始地址(16 位）
            WriteData16(xe); // X 结束地址(16 位）
            WriteCom(0x2b); // 设置Y 开始及结束的地址
            WriteData16(y); // Y 开始地址(16 位）
            WriteData16(ye); // Y 结束地址(16 位）
            WriteCom(0x2c); // 写数据开始
        }

        private void WriteData16(ushort data)
        {
            WriteData((byte)(data >> 8));
            WriteData((byte)data);
        }

        private void WriteCom(byte data)
        {
            _manager.WritePin(_pinCs, 0);
            _manager.WritePin(_pinRs, 0);
            _manager.WritePin(_pinRd, 1);

            for (var i = 0; i < 8; i++)
            {
                _manager.WritePin(_pinData[i], (data & 1 << i) != 0);
            }

            _manager.WritePin(_pinWr, 0);
            _manager.WritePin(_pinWr, 1);
            _manager.WritePin(_pinCs, 1);
        }

        private void WriteData(byte data)
        {
            _manager.WritePin(_pinCs, 0);
            _manager.WritePin(_pinRs, 1);
            _manager.WritePin(_pinRd, 1);

            for (var i = 0; i < 8; i++)
            {
                _manager.WritePin(_pinData[i], (data & 1 << i) != 0);
            }

            _manager.WritePin(_pinWr, 0);
            _manager.WritePin(_pinWr, 1);
            _manager.WritePin(_pinCs, 1);
        }
    }
}