﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Text;

namespace OledSsd1306
{
    internal class BitmapMaker : IDisposable
    {
        private readonly PrivateFontCollection _fontCollection = new PrivateFontCollection();
        private readonly Bitmap _bmp;
        private readonly Graphics _graphics;

        private Font _font;
        private byte _filter = 16;

        private readonly int _width;
        private readonly int _height;

        private readonly OledSsd1306Driver _target;

        private readonly Stopwatch _sw = new Stopwatch();

        public BitmapMaker(OledSsd1306Driver target, int fontSize = 10, int width = 128, int height = 32)
        {
            _target = target;

            _width = width;
            _height = height;

            _font = new Font("文泉驿点阵正黑", fontSize, GraphicsUnit.Pixel);
            _bmp = new Bitmap(width, height, PixelFormat.Format32bppRgb);
            _graphics = Graphics.FromImage(_bmp);
        }

        public void SetFontSize(int size)
        {
            _font.Dispose();
            _font = new Font(_fontCollection.Families[0], size);
        }

        public void SetFilter(byte value)
        {
            _filter = value;
        }

        public void Make(string str, bool inv = false, bool box = false)
        {
            //_sw.Restart();
            Color bg;
            Brush fgb;
            Pen fgp;

            if (inv)
            {
                bg = Color.White;
                fgb = Brushes.Black;
                fgp = Pens.Black;
            }
            else
            {
                bg = Color.Black;
                fgb = Brushes.White;
                fgp = Pens.White;
            }

            _graphics.Clear(bg);

            if (box) _graphics.DrawRectangle(fgp, 1, 1, _width - 3, _height - 3);

            _graphics.DrawString(str, _font, fgb
                , new RectangleF(0, 0, _bmp.Width, _bmp.Height)
                , new StringFormat
                {
                    Alignment = StringAlignment.Center,
                    LineAlignment = StringAlignment.Center
                });

            //_sw.Stop();
            //Console.WriteLine($"#DEBUG BM RENDER MS:{_sw.ElapsedMilliseconds}");

            //_sw.Restart();

            var lck = _bmp.LockBits(new Rectangle(0, 0, _width, _height), ImageLockMode.ReadOnly, PixelFormat.Format32bppRgb);
            unsafe
            {
                for (var y = 0; y < _height; y++)
                {
                    var prow = (int*)(lck.Scan0 + lck.Stride * y);
                    for (var x = 0; x < _width; x++)
                    {
                        _target.SetPixel(x, y, (prow[x] & 0xFF) > _filter);
                    }
                }
            }
            _bmp.UnlockBits(lck);

            //_sw.Stop();
            //Console.WriteLine($"#DEBUG BM BLIT MS:{_sw.ElapsedMilliseconds}");

            _target.Update();
        }

        public void Dispose()
        {
            _font?.Dispose();
            _fontCollection?.Dispose();
            _graphics?.Dispose();
            _bmp?.Dispose();
        }
    }
}