﻿using System;
using System.IO;
using System.Runtime.InteropServices;

namespace OledSsd1306
{
    internal class Raspiic : IDisposable
    {
        private const int IIC_SLAVE = 0x0703;
        private const int IIC_SLAVE_FORCE = 0x0706;

        private readonly string _iicBusFile;
        private readonly int _device;

        private FileStream _fdWrite;

        [DllImport("libc", EntryPoint = "ioctl", SetLastError = true)]
        private static extern int KernelIoCtrl(IntPtr fd, int request, int data);

        public Raspiic(int device, int bus = 1)
        {
            _device = device;
            _iicBusFile = $"/dev/i2c-{bus}";
        }

        public void Init()
        {
            _fdWrite = new FileStream(_iicBusFile, FileMode.Open, FileAccess.Write);
            KernelIoCtrl(_fdWrite.SafeFileHandle.DangerousGetHandle(), IIC_SLAVE, _device);
        }

        public void Write(params byte[] data) => _fdWrite.Write(data, 0, data.Length);

        public void Close() => _fdWrite.Close();

        public void Dispose() => Close();
    }
}