﻿using System;
using System.IO;
using WiringPi;

namespace WiringPiHc595LedArray
{
    internal class Program
    {
        private const int PwmMax = 1000;

        private const int PinButtonLeft = 17;
        private const int PinButtonMiddle = 24;
        private const int PinButtonRight = 25;

        private const int PinPwm = 18;

        //REF: swf.com.tw/?p=715
        private const int PinBit = 23;
        private const int PinClk = 22;
        private const int PinFlu = 27;

        private static void Main(string[] args)
        {
            var ci = File.ReadAllLines("/proc/cpuinfo");
            foreach (var s in ci) Console.WriteLine(s);

            var ret = Init.WiringPiSetupGpio();
            if (ret == -1)
            {
                Console.WriteLine("Init failed: {0}", ret);
                return;
            }

            GPIO.pinMode(PinButtonLeft, (int)GPIO.GPIOpinmode.Input);
            GPIO.pinMode(PinButtonMiddle, (int)GPIO.GPIOpinmode.Input);
            GPIO.pinMode(PinButtonRight, (int)GPIO.GPIOpinmode.Input);

            GPIO.pinMode(PinBit, (int)GPIO.GPIOpinmode.Output);
            GPIO.pinMode(PinClk, (int)GPIO.GPIOpinmode.Output);
            GPIO.pinMode(PinFlu, (int)GPIO.GPIOpinmode.Output);

            GPIO.pinMode(PinPwm, (int)GPIO.GPIOpinmode.PWMOutput);
            GPIO.pwmSetClock(2);
            GPIO.pwmSetRange(PwmMax);
            GPIO.pwmWrite(PinPwm, 990);


            GPIO.digitalWrite(PinFlu, (int)GPIO.GPIOpinvalue.Low);
            ShiftOut(false, 0xff);
            ShiftOut(false, 0xff);
            GPIO.digitalWrite(PinFlu, (int)GPIO.GPIOpinvalue.High);
            Console.WriteLine();

            var num = 0;

            while (true)
            {
                var l = GPIO.digitalRead(PinButtonLeft);
                var m = GPIO.digitalRead(PinButtonMiddle);
                var r = GPIO.digitalRead(PinButtonRight);

                Console.Write($"{DateTime.Now:HH:mm:ss.fff} L={l} M={m} R={r}");

                GPIO.digitalWrite(PinFlu, (int)GPIO.GPIOpinvalue.Low);
                ShiftOut(false, (byte)++num);
                GPIO.digitalWrite(PinFlu, (int)GPIO.GPIOpinvalue.High);

                Console.WriteLine();

                Timing.delay(20);
            }
        }

        private static void ShiftOut(bool lsbFirst, byte val)
        {
            Console.Write(" ShiftOut:");
            for (var i = 0; i < 8; i++)
            {
                bool v;

                if (lsbFirst) v = 0 != (val & (1 << i));
                else v = 0 != (val & (1 << (7 - i)));

                GPIO.digitalWrite(PinBit, v ? 1 : 0);
                GPIO.digitalWrite(PinClk, (int)GPIO.GPIOpinvalue.High);

                Console.Write(v ? 1 : 0);

                GPIO.digitalWrite(PinClk, (int)GPIO.GPIOpinvalue.Low);
            }
        }
    }
}